@@ -2,6 +2,7 @@ pub(crate) mod command;
 pub(crate) mod fake;
 pub(crate) mod file;
 pub(crate) mod git;
+pub(crate) mod installergo;
 
 use std::{cell::Cell, convert::TryFrom, fmt, time::Instant};
 
@@ -13,6 +14,7 @@ use command::Command;
 use fake::Fake;
 use file::File;
 use git::Git;
+use installergo::InstallerGo;
 
 #[derive(Debug, ThisError)]
 pub enum Error {
@@ -37,6 +39,11 @@ pub enum Error {
         source: git::Error,
     },
     #[error(transparent)]
+    InstallerGoJob {
+        #[from]
+        source: installergo::Error,
+    },
+    #[error(transparent)]
     ParseToml {
         #[from]
         source: toml::de::Error,
@@ -95,6 +102,7 @@ impl Execute for Job {
             Spec::Fake(j) => j.execute().map_err(|e| Error::FakeJob { source: e }),
             Spec::File(j) => j.execute().map_err(|e| Error::FileJob { source: e }),
             Spec::Git(j) => j.execute().map_err(|e| Error::GitJob { source: e }),
+            Spec::InstallerGo(j) => j.execute().map_err(|e| Error::InstallerGoJob { source: e }),
         };
         self.history.updated.set(Instant::now());
         result
@@ -146,6 +154,7 @@ pub enum Spec {
     Fake(Fake),
     File(File),
     Git(Git),
+    InstallerGo(InstallerGo),
 }
 impl fmt::Display for Spec {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -154,6 +163,7 @@ impl fmt::Display for Spec {
             Spec::Fake(j) => write!(f, "{}", j),
             Spec::File(j) => write!(f, "{}", j),
             Spec::Git(j) => write!(f, "{}", j),
+            Spec::InstallerGo(j) => write!(f, "{}", j),
         }
     }
 }
@@ -266,11 +276,12 @@ impl Status {
 
 #[cfg(test)]
 mod tests {
-    use std::{path::PathBuf, time::Duration};
+    use std::{collections::HashMap, path::PathBuf, time::Duration};
 
     use git_url::Url;
 
     use file::FileState;
+    use installergo::DesiredState;
 
     use super::*;
 
@@ -280,24 +291,28 @@ mod tests {
             [[jobs]]
             name = "run something"
             type = "command"
-            command = "something"
             argv = [ "foo" ]
+            command = "something"
+            env = { VARIABLE = "value" }
             "#;
 
         let got = Main::try_from(input)?;
 
+        let mut env: HashMap<String, String> = HashMap::new();
+        env.insert(String::from("VARIABLE"), String::from("value"));
         let want = Main {
             jobs: vec![Job {
-                history: History::default(),
                 metadata: Metadata {
                     name: Some(String::from("run something")),
                     ..Default::default()
                 },
                 spec: Spec::Command(Command {
                     argv: Some(vec![String::from("foo")]),
+                    env: Some(env),
                     command: String::from("something"),
                     ..Default::default()
                 }),
+                ..Default::default()
             }],
         };
 
@@ -322,9 +337,6 @@ mod tests {
 
         let want = Main {
             jobs: vec![Job {
-                history: History {
-                    ..Default::default()
-                },
                 metadata: Metadata {
                     name: Some(String::from("fake")),
                     ..Default::default()
@@ -333,6 +345,7 @@ mod tests {
                     sleep_ms: Some(Duration::from_millis(5000)),
                     ..Default::default()
                 }),
+                ..Default::default()
             }],
         };
 
@@ -358,9 +371,6 @@ mod tests {
 
         let want = Main {
             jobs: vec![Job {
-                history: History {
-                    ..Default::default()
-                },
                 metadata: Metadata {
                     name: Some(String::from("mkdir /tmp")),
                     ..Default::default()
@@ -370,6 +380,7 @@ mod tests {
                     state: FileState::Directory,
                     ..Default::default()
                 }),
+                ..Default::default()
             }],
         };
 
@@ -395,9 +406,6 @@ mod tests {
 
         let want = Main {
             jobs: vec![Job {
-                history: History {
-                    ..Default::default()
-                },
                 metadata: Metadata {
                     name: Some(String::from("clone tuning source repo")),
                     ..Default::default()
@@ -408,6 +416,37 @@ mod tests {
                         .expect("unable to parse test URL"),
                     ..Default::default()
                 }),
+                ..Default::default()
+            }],
+        };
+
+        assert_eq!(got.jobs.len(), 1);
+        // TODO: find an approach that enables reliable `history` comparison
+        assert_eq!(got.jobs[0].metadata, want.jobs[0].metadata);
+        assert_eq!(got.jobs[0].spec, want.jobs[0].spec);
+
+        Ok(())
+    }
+
+    #[test]
+    fn install_go_toml() -> std::result::Result<(), Error> {
+        let input = r#"
+            [[jobs]]
+            type = "installergo"
+            packages = [ "gitlab.com/jokeyrhyme/dotfiles/barista" ]
+            state = "latest"
+            "#;
+
+        let got = Main::try_from(input)?;
+
+        let want = Main {
+            jobs: vec![Job {
+                spec: Spec::InstallerGo(InstallerGo {
+                    packages: vec![String::from("gitlab.com/jokeyrhyme/dotfiles/barista")],
+                    state: DesiredState::Latest,
+                    ..Default::default()
+                }),
+                ..Default::default()
             }],
         };
 
